@@ -26,7 +26,7 @@ from pytensor.tensor.basic import (
     as_tensor,
     cast,
     concatenate,
-    extract_constant,
+    get_scalar_constant_value,
     get_underlying_scalar_constant_value,
     register_infer_shape,
     switch,
@@ -85,7 +85,7 @@ from pytensor.tensor.subtensor import (
     inc_subtensor,
     indices_from_subtensor,
 )
-from pytensor.tensor.type import TensorType
+from pytensor.tensor.type import TensorType, integer_dtypes
 from pytensor.tensor.type_other import NoneTypeT, SliceConstant, SliceType
 from pytensor.tensor.variable import TensorConstant, TensorVariable
 
@@ -390,8 +390,8 @@ def local_useless_slice(fgraph, node):
         start = s.start
         stop = s.stop
 
-        if start is not None and extract_constant(
-            start, only_process_constants=True
+        if start is not None and get_scalar_constant_value(
+            start, only_process_constants=True, raise_not_constant=False
         ) == (0 if positive_step else -1):
             change_flag = True
             start = None
@@ -399,7 +399,9 @@ def local_useless_slice(fgraph, node):
         if (
             stop is not None
             and x.type.shape[dim] is not None
-            and extract_constant(stop, only_process_constants=True)
+            and get_scalar_constant_value(
+                stop, only_process_constants=True, raise_not_constant=False
+            )
             == (x.type.shape[dim] if positive_step else -x.type.shape[dim] - 1)
         ):
             change_flag = True
@@ -889,7 +891,10 @@ def local_useless_inc_subtensor(fgraph, node):
         and e.stop is None
         and (
             e.step is None
-            or extract_constant(e.step, only_process_constants=True) == -1
+            or get_scalar_constant_value(
+                e.step, only_process_constants=True, raise_not_constant=False
+            )
+            == -1
         )
         for e in idx_cst
     ):
@@ -994,7 +999,7 @@ def local_useless_subtensor(fgraph, node):
         if isinstance(idx.stop, int | np.integer):
             length_pos_data = sys.maxsize
             try:
-                length_pos_data = get_underlying_scalar_constant_value(
+                length_pos_data = get_scalar_constant_value(
                     length_pos, only_process_constants=True
                 )
             except NotScalarConstantError:
@@ -1059,7 +1064,7 @@ def local_useless_AdvancedSubtensor1(fgraph, node):
 
     # get length of the indexed tensor along the first axis
     try:
-        length = get_underlying_scalar_constant_value(
+        length = get_scalar_constant_value(
             shape_of[node.inputs[0]][0], only_process_constants=True
         )
     except NotScalarConstantError:
@@ -1490,7 +1495,10 @@ def local_adv_sub1_adv_inc_sub1(fgraph, node):
         and
         # Don't use only_process_constants=True. We need to
         # investigate Alloc of 0s but with non constant shape.
-        extract_constant(x, elemwise=False) != 0
+        get_underlying_scalar_constant_value(
+            x, elemwise=False, raise_not_constant=False
+        )
+        != 0
     ):
         return
 
@@ -1728,7 +1736,7 @@ def local_join_subtensors(fgraph, node):
     axis, tensors = node.inputs[0], node.inputs[1:]
 
     try:
-        axis = get_underlying_scalar_constant_value(axis)
+        axis = get_scalar_constant_value(axis)
     except NotScalarConstantError:
         return
 
@@ -1789,12 +1797,7 @@ def local_join_subtensors(fgraph, node):
             if step is None:
                 continue
             try:
-                if (
-                    get_underlying_scalar_constant_value(
-                        step, only_process_constants=True
-                    )
-                    != 1
-                ):
+                if get_scalar_constant_value(step, only_process_constants=True) != 1:
                     return None
             except NotScalarConstantError:
                 return None
@@ -1981,7 +1984,7 @@ def ravel_multidimensional_bool_idx(fgraph, node):
 
     if any(
         (
-            (isinstance(idx.type, TensorType) and idx.type.dtype.startswith("int"))
+            (isinstance(idx.type, TensorType) and idx.type.dtype in integer_dtypes)
             or isinstance(idx.type, NoneTypeT)
         )
         for idx in idxs
@@ -2052,7 +2055,7 @@ def ravel_multidimensional_int_idx(fgraph, node):
     int_idxs = [
         (i, idx)
         for i, idx in enumerate(idxs)
-        if (isinstance(idx.type, TensorType) and idx.dtype.startswith("int"))
+        if (isinstance(idx.type, TensorType) and idx.dtype in integer_dtypes)
     ]
 
     if len(int_idxs) != 1:
